# -*- coding: utf-8 -*-
"""Provides functions to calculate and manipulate
Axis-Aligned Minimum Bounding Boxes (AAMBB).

AAMBB are a simple 3D rectangle with no orientation.
It is up to the user to provide translation.
AAMBB differ from AABB in that they allow for the
content to rotate freely and still be within the AAMBB.

An AAMBB is represented in the same way an AABB is;
a array of 2 x 3D vectors.
The first vector represents the minimum extent.
The second vector represents the maximum extent.

Note that because the AAMBB set's it's dimensions using
the vector length of any points set within it, the user
should be careful to avoid adding the AAMBB to itself
or the AAMBB will continue to grow.

TODO: add transform( matrix )
TODO: add point_within_aabb
TODO: use point_within_aabb for unit tests
"""
from __future__ import absolute_import, division, print_function
import numpy as np
from . import aabb, vector
from .utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays


class index:
    #: The index of the minimum vector within the AAMBB
    minimum = 0

    #: The index of the minimum vector within the AAMBB
    maximum = 1


def create_zeros(dtype=None):
    return np.zeros((2,3), dtype=dtype)

@parameters_as_numpy_arrays('min', 'max')
def create_from_bounds(min, max, dtype=None):
    """Creates an AAMBB using the specified minimum
    and maximum values.
    """
    dtype = dtype or min.dtype
    # stack our bounds together and add them as points
    points = np.vstack((min, max))
    return create_from_points(points, dtype)

@parameters_as_numpy_arrays('points')
def create_from_points(points, dtype=None):
    """Creates an AAMBB from the list of specified points.

    Points must be a 2D list. Ie::
        numpy.array([
            [ x, y, z ],
            [ x, y, z ],
            ])
    """
    dtype = dtype or points.dtype

    # convert any negative values to positive
    abs_points = np.absolute(points)

    # find the length of this vector
    length = np.amax(vector.length(abs_points))

    # our AAMBB extends from +length to -length
    # in all directions
    return np.array(
        [
            [-length,-length,-length ],
            [ length, length, length ]
        ],
        dtype=dtype
    )

@all_parameters_as_numpy_arrays
def create_from_aabbs(aabbs, dtype=None):
    """Creates an AAMBB from a list of existing AABBs.

    AABBs must be a 2D list. Ie::
        numpy.array([
            AABB,
            AABB,
            ])
    """
    dtype = dtype or aabbs.dtype
    # reshape the AABBs as a series of points
    points = aabbs.reshape((-1, 3))

    return create_from_points(points, dtype=dtype)

@parameters_as_numpy_arrays('bb')
def add_points(bb, points):
    """Extends an AAMBB to encompass a list
    of points.

    It should be noted that this ensures that
    the encompassed points can rotate freely.
    Calling this using the min / max points from
    the AAMBB will create an even bigger AAMBB.
    """
    # add our AABB to the list of points
    values = np.vstack((points, bb[0], bb[1]))

    # convert any negative values to positive
    abs_points = np.absolute(values)

    # extract the maximum extent as a vector
    #vec = np.amax(abs_points, axis=0)

    # find the length of this vector
    #length = vector.length(vec)
    length = np.amax(vector.length(abs_points))

    # our AAMBB extends from +length to -length
    # in all directions
    return np.array(
        [
            [-length,-length,-length ],
            [ length, length, length ]
        ],
        dtype=bb.dtype
    )

@parameters_as_numpy_arrays('bbs')
def add_aabbs(bb, bbs):
    """Extend an AAMBB to encompass a list
    of other AABBs or AAMBBs.

    It should be noted that this ensures that
    the encompassed AABBs can rotate freely.
    Using the AAMBB itself in this calculation
    will create an event bigger AAMBB.
    """
    # reshape the AABBs as a series of points
    points = bbs.reshape((-1, 3))

    # use the add_points
    return add_points(bb, points)

def centre_point(bb):
    """Returns the centre point of the AABB.
    This should always be [0.0, 0.0, 0.0]
    """
    return aabb.centre_point(bb)

def minimum(bb):
    """Returns the minimum point of the AABB.
    """
    return aabb.minimum(bb)

def maximum(bb):
    """Returns the maximum point of the AABB.
    """
    return aabb.maximum(bb)

def clamp_points(bb, points):
    """Takes a list of points and modifies them to
    fit within the AABB.
    """
    # use the same function as present in AABB
    aabb.clamp_points(bb, points)




from __future__ import absolute_import
try:
    import unittest2 as unittest
except:
    import unittest


class test_procedural_examples(unittest.TestCase):
    def test_procedural_examples(self):
        from pyrr import quaternion, matrix44, vector3
        import numpy as np

        point = vector3.create(1.,2.,3.)
        orientation = quaternion.create()
        translation = vector3.create()
        scale = vector3.create(1,1,1)

        # translate along X by 1
        translation += [1.0, 0.0, 0.0]

        # rotate about Y by pi/2
        rotation = quaternion.create_from_y_rotation(np.pi / 2.0)
        orientation = quaternion.cross(rotation, orientation)

        # create a matrix
        # start our matrix off using the scale
        matrix = matrix44.create_from_scale(scale)

        # apply our orientation
        orientation = matrix44.create_from_quaternion(orientation)
        matrix = matrix44.multiply(matrix, orientation)

        # apply our translation
        translation_matrix = matrix44.create_from_translation(translation)
        matrix = matrix44.multiply(matrix, translation_matrix)

        # transform our point by the matrix
        point = matrix44.apply_to_vector(matrix, point)



# -*- coding: utf-8 -*-
"""Provide functions for the creation and manipulation of Quaternions.
"""
from __future__ import absolute_import, division, print_function
import numpy as np
from . import vector, vector3, vector4
from .utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays


class index:
    #: The index of the X value within the quaternion
    x = 0

    #: The index of the Y value within the quaternion
    y = 1

    #: The index of the Z value within the quaternion
    z = 2

    #: The index of the W value within the quaternion
    w = 3


def create(x=0., y=0., z=0., w=1., dtype=None):
    return np.array([x, y, z, w], dtype=dtype)

def create_from_x_rotation(theta, dtype=None):
    thetaOver2 = theta * 0.5

    return np.array(
        [
            np.sin(thetaOver2),
            0.0,
            0.0,
            np.cos(thetaOver2)
        ],
        dtype=dtype
    )

def create_from_y_rotation(theta, dtype=None):
    thetaOver2 = theta * 0.5

    return np.array(
        [
            0.0,
            np.sin(thetaOver2),
            0.0,
            np.cos(thetaOver2)
        ],
        dtype=dtype
    )

def create_from_z_rotation(theta, dtype=None):
    thetaOver2 = theta * 0.5

    return np.array(
        [
            0.0,
            0.0,
            np.sin(thetaOver2),
            np.cos(thetaOver2)
        ],
        dtype=dtype
    )

@parameters_as_numpy_arrays('axis')
def create_from_axis_rotation(axis, theta, dtype=None):
    dtype = dtype or axis.dtype
    # make sure the vector is normalised
    if not np.isclose(np.linalg.norm(axis), 1.):
        axis = vector.normalise(axis)

    thetaOver2 = theta * 0.5
    sinThetaOver2 = np.sin(thetaOver2)

    return np.array(
        [
            sinThetaOver2 * axis[0],
            sinThetaOver2 * axis[1],
            sinThetaOver2 * axis[2],
            np.cos(thetaOver2)
        ],
        dtype=dtype
    )

@parameters_as_numpy_arrays('mat')
def create_from_matrix(mat, dtype=None):
    # http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm
    # optimised "alternative version" does not produce correct results
    # see issue #42
    dtype = dtype or mat.dtype

    trace = mat[0][0] + mat[1][1] + mat[2][2]
    if trace > 0:
        s = 0.5 / np.sqrt(trace + 1.0)
        qx = (mat[2][1] - mat[1][2]) * s
        qy = (mat[0][2] - mat[2][0]) * s
        qz = (mat[1][0] - mat[0][1]) * s
        qw = 0.25 / s
    elif mat[0][0] > mat[1][1] and mat[0][0] > mat[2][2]:
        s = 2.0 * np.sqrt(1.0 + mat[0][0] - mat[1][1] - mat[2][2])
        qx = 0.25 * s
        qy = (mat[0][1] + mat[1][0]) / s
        qz = (mat[0][2] + mat[2][0]) / s
        qw = (mat[2][1] - mat[1][2]) / s
    elif mat[1][1] > mat[2][2]:
        s = 2.0 * np.sqrt(1.0 + mat[1][1] - mat[0][0] - mat[2][2])
        qx = (mat[0][1] + mat[1][0]) / s
        qy = 0.25 * s
        qz = (mat[1][2] + mat[2][1]) / s
        qw = (mat[0][2] - mat[2][0]) / s
    else:
        s = 2.0 * np.sqrt(1.0 + mat[2][2] - mat[0][0] - mat[1][1])
        qx = (mat[0][2] + mat[2][0]) / s
        qy = (mat[1][2] + mat[2][1]) / s
        qz = 0.25 * s
        qw = (mat[1][0] - mat[0][1]) / s

    quat = np.array([qx, qy, qz, qw], dtype=dtype)
    return quat

@parameters_as_numpy_arrays('eulers')
def create_from_eulers(eulers, dtype=None):
    """Creates a quaternion from a set of Euler angles.

    Eulers are an array of length 3 in the following order::
        [yaw, pitch, roll]
    """
    dtype = dtype or eulers.dtype

    pitch, yaw, roll = eulers

    halfPitch = pitch * 0.5
    sP = np.sin(halfPitch)
    cP = np.cos(halfPitch)

    halfRoll = roll * 0.5
    sR = np.sin(halfRoll)
    cR = np.cos(halfRoll)

    halfYaw = yaw * 0.5
    sY = np.sin(halfYaw)
    cY = np.cos(halfYaw)

    return np.array(
        [
            # x = -cy * sp * cr - sy * cp * sr
            (-cY * sP * cR) - (sY * cP * sR),
            # y = cy * sp * sr - sy * cp * cr
            (cY * sP * sR) - (sY * cP * cR),
            # z = sy * sp * cr - cy * cp * sr
            (sY * sP * cR) - (cY * cP * sR),
            # w = cy * cp * cr + sy * sp * sr
            (cY * cP * cR) + (sY * sP * sR),
        ],
        dtype=dtype
    )

@parameters_as_numpy_arrays('axis')
def create_from_inverse_of_eulers(eulers, dtype=None):
    """Creates a quaternion from the inverse of a set of Euler angles.

    Eulers are an array of length 3 in the following order::
        [yaw, pitch, roll]
    """
    dtype = dtype or eulers.dtype

    pitch, roll, yaw = euler.pitch(eulers), euler.roll(eulers), euler.yaw(eulers)

    halfRoll = roll * 0.5
    sinRoll = np.sin(halfRoll)
    cosRoll = np.cos(halfRoll)

    halfPitch = pitch * 0.5
    sinPitch = np.sin(halfPitch)
    cosPitch = np.cos(halfPitch)

    halfYaw = yaw * 0.5
    sinYaw = np.sin(halfYaw)
    cosYaw = np.cos(halfYaw)

    return np.array(
        [
            # x = cy * sp * cr + sy * cp * sr
            (cosYaw * sinPitch * cosRoll) + (sinYaw * cosPitch * sinRoll),
            # y = -cy * sp * sr + sy * cp * cr
            (-cosYaw * sinPitch * sinRoll) + (sinYaw * cosPitch * cosRoll),
            # z = -sy * sp * cr + cy * cp * sr
            (-sinYaw * sinPitch * cosRoll) + (cosYaw * cosPitch * sinRoll),
            # w = cy * cp * cr + sy * sp * sr
            (cosYaw * cosPitch * cosRoll) + (sinYaw * sinPitch * sinRoll)
        ],
        dtype=dtype
    )

@all_parameters_as_numpy_arrays
def cross(quat1, quat2):
    """Returns the cross-product of the two quaternions.

    Quaternions are **not** communicative. Therefore, order is important.

    This is NOT the same as a vector cross-product.
    Quaternion cross-product is the equivalent of matrix multiplication.
    """
    q1x, q1y, q1z, q1w = quat1
    q2x, q2y, q2z, q2w = quat2

    return np.array(
        [
             q1x * q2w + q1y * q2z - q1z * q2y + q1w * q2x,
            -q1x * q2z + q1y * q2w + q1z * q2x + q1w * q2y,
             q1x * q2y - q1y * q2x + q1z * q2w + q1w * q2z,
            -q1x * q2x - q1y * q2y - q1z * q2z + q1w * q2w,
        ],
        dtype=quat1.dtype
    )

def is_zero_length(quat):
    """Checks if a quaternion is zero length.

    :param numpy.array quat: The quaternion to check.
    :rtype: boolean.
    :return: True if the quaternion is zero length, otherwise False.
    """
    return quat[0] == quat[1] == quat[2] == quat[3] == 0.0

def is_non_zero_length(quat):
    """Checks if a quaternion is not zero length.

    This is the opposite to 'is_zero_length'.
    This is provided for readability.

    :param numpy.array quat: The quaternion to check.
    :rtype: boolean
    :return: False if the quaternion is zero length, otherwise True.

    .. seealso:: is_zero_length
    """
    return not is_zero_length(quat)

def squared_length(quat):
    """Calculates the squared length of a quaternion.

    Useful for avoiding the performanc penalty of
    the square root function.

    :param numpy.array quat: The quaternion to measure.
    :rtype: float, numpy.array
    :return: If a 1d array was passed, it will be a scalar.
        Otherwise the result will be an array of scalars with shape
        vec.ndim with the last dimension being size 1.
    """
    return vector4.squared_length(quat)

def length(quat):
    """Calculates the length of a quaternion.

    :param numpy.array quat: The quaternion to measure.
    :rtype: float, numpy.array
    :return: If a 1d array was passed, it will be a scalar.
        Otherwise the result will be an array of scalars with shape
        vec.ndim with the last dimension being size 1.
    """
    return vector4.length(quat)

def normalise(quat):
    """Ensure a quaternion is unit length (length ~= 1.0).

    The quaternion is **not** changed in place.

    :param numpy.array quat: The quaternion to normalise.
    :rtype: numpy.array
    :return: The normalised quaternion(s).
    """
    return vector4.normalise(quat)

def rotation_angle(quat):
    """Calculates the rotation around the quaternion's axis.

    :param numpy.array quat: The quaternion.
    :rtype: float.
    :return: The quaternion's rotation about the its axis in radians.
    """
    # extract the W component
    thetaOver2 = np.arccos(quat[3])
    return thetaOver2 * 2.0

@all_parameters_as_numpy_arrays
def rotation_axis(quat):
    """Calculates the axis of the quaternion's rotation.

    :param numpy.array quat: The quaternion.
    :rtype: numpy.array.
    :return: The quaternion's rotation axis.
    """
    # extract W component
    sinThetaOver2Sq = 1.0 - (quat[3] ** 2)

    # check for zero before we sqrt
    if sinThetaOver2Sq <= 0.0:
        # identity quaternion or numerical imprecision.
        # return a valid vector
        # we'll treat -Z as the default
        return np.array([0.0, 0.0, -1.0], dtype=quat.dtype)

    oneOverSinThetaOver2 = 1.0 / np.sqrt(sinThetaOver2Sq)

    # we use the x,y,z values
    return np.array(
        [
            quat[0] * oneOverSinThetaOver2,
            quat[1] * oneOverSinThetaOver2,
            quat[2] * oneOverSinThetaOver2
        ],
        dtype=quat.dtype
    )

def dot(quat1, quat2):
    """Calculate the dot product of quaternions.

    This is the same as a vector dot product.

    :param numpy.array quat1: The first quaternion(s).
    :param numpy.array quat2: The second quaternion(s).
    :rtype: float, numpy.array
    :return: If a 1d array was passed, it will be a scalar.
        Otherwise the result will be an array of scalars with shape
        vec.ndim with the last dimension being size 1.
    """
    return vector4.dot(quat1, quat2)

@all_parameters_as_numpy_arrays
def conjugate(quat):
    """Calculates a quaternion with the opposite rotation.

    :param numpy.array quat: The quaternion.
    :rtype: numpy.array.
    :return: A quaternion representing the conjugate.
    """

    # invert x,y,z and leave w as is
    return np.array(
        [
            -quat[0],
            -quat[1],
            -quat[2],
            quat[3]
        ],
        dtype=quat.dtype
    )

@parameters_as_numpy_arrays('quat')
def power(quat, exponent):
    """Multiplies the quaternion by the exponent.

    The quaternion is **not** changed in place.

    :param numpy.array quat: The quaternion.
    :param float scalar: The exponent.
    :rtype: numpy.array.
    :return: A quaternion representing the original quaternion
        to the specified power.
    """
    # check for identify quaternion
    if np.fabs(quat[3]) > 0.9999:
        # assert for the time being
        assert False
        print("rotation axis was identity")

        return quat

    alpha = np.arccos(quat[3])
    newAlpha = alpha * exponent
    multi = np.sin(newAlpha) / np.sin(alpha)

    return np.array(
        [
            quat[0] * multi,
            quat[1] * multi,
            quat[2] * multi,
            np.cos(newAlpha)
        ],
        dtype=quat.dtype
    )

def inverse(quat):
    """Calculates the inverse quaternion.

    The inverse of a quaternion is defined as
    the conjugate of the quaternion divided
    by the magnitude of the original quaternion.

    :param numpy.array quat: The quaternion to invert.
    :rtype: numpy.array.
    :return: The inverse of the quaternion.
    """
    return conjugate(quat) / length(quat)

@all_parameters_as_numpy_arrays
def negate(quat):
    """Calculates the negated quaternion.

    This is essentially the quaternion * -1.0.

    :param numpy.array quat: The quaternion.
    :rtype: numpy.array
    :return: The negated quaternion.
    """
    return quat * -1.0

def is_identity(quat):
    return np.allclose(quat, [0.,0.,0.,1.])

@all_parameters_as_numpy_arrays
def apply_to_vector(quat, vec):
    """Rotates a vector by a quaternion.

    :param numpy.array quat: The quaternion.
    :param numpy.array vec: The vector.
    :rtype: numpy.array
    :return: The vector rotated by the quaternion.
    :raise ValueError: raised if the vector is an unsupported size

    .. seealso:: http://content.gpwiki.org/index.php/OpenGL:Tutorials:Using_Quaternions_to_represent_rotation
    """
    def apply(quat, vec4):
        result = cross(quat, cross(vec4, conjugate(quat)))
        return result

    if vec.size == 3:
        # convert to vector4
        # ignore w component by setting it to 0.
        vec = np.array([vec[0], vec[1], vec[2], 0.0], dtype=vec.dtype)
        vec = apply(quat, vec)
        vec = vec[:3]
        return vec
    elif vec.size == 4:
        vec = apply(quat, vec)
        return vec
    else:
        raise ValueError("Vector size unsupported")



# -*- coding: utf-8 -*-
"""Provide functions for the creation and manipulation of Euler angles.

Eulers represent 3 rotations: Pitch, Roll and Yaw.

Eulers are represented using a numpy.array of shape (3,).
"""
from __future__ import absolute_import, division, print_function
import numpy as np


class index:
    """Defines the indices used to store the Euler values in the numpy array.
    """
    #: The index of the roll value within the euler.
    roll = 0

    #: The index of the pitch value within the euler.
    pitch = 1

    #: The index of the yaw value within the euler.
    yaw = 2


def create(roll=0., pitch=0., yaw=0., dtype=None):
    """Creates an array storing the specified euler angles.

    Input values are in radians.

    :param float pitch: The pitch in radians.
    :param float roll: The roll in radians.
    :param float yaw: The yaw in radians.
    :rtype: numpy.array
    """
    return np.array((roll, pitch, yaw), dtype=dtype)


def create_from_x_rotation(theta, dtype=None):
    return np.array([theta, 0., 0.], dtype=dtype)


def create_from_y_rotation(theta, dtype=None):
    return np.array([0., theta, 0.], dtype=dtype)


def create_from_z_rotation(theta, dtype=None):
    return np.array([0., 0., theta], dtype=dtype)


def roll(eulers):
    """Extracts the roll value from the euler.

    :rtype: float.
    """
    return eulers[0]


def yaw(eulers):
    """Extracts the yaw value from the euler.

    :rtype: float.
    """
    return eulers[2]


def pitch(eulers):
    """Extracts the pitch value from the euler.

    :rtype: float.
    """
    return eulers[1]



from __future__ import absolute_import, division, print_function
try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import quaternion, matrix44, matrix33, euler


class test_matrix_quaternion(unittest.TestCase):
    def test_m44_q_equivalence(self):
        """Test for equivalance of matrix and quaternion rotations.

        Create a matrix and quaternion, rotate each by the same values
        then convert matrix<->quaternion and check the results are the same.
        """
        m = matrix44.create_from_x_rotation(np.pi / 2.)
        mq = quaternion.create_from_matrix(m)

        q = quaternion.create_from_x_rotation(np.pi / 2.)
        qm = matrix44.create_from_quaternion(q)

        self.assertTrue(np.allclose(np.dot([1., 0., 0., 1.], m), [1., 0., 0., 1.]))
        self.assertTrue(np.allclose(np.dot([1., 0., 0., 1.], qm), [1., 0., 0., 1.]))

        self.assertTrue(np.allclose(quaternion.apply_to_vector(q, [1., 0., 0., 1.]), [1., 0., 0., 1.]))
        self.assertTrue(np.allclose(quaternion.apply_to_vector(mq, [1., 0., 0., 1.]), [1., 0., 0., 1.]))

        np.testing.assert_almost_equal(q, mq, decimal=5)
        np.testing.assert_almost_equal(m, qm, decimal=5)

    def test_euler_equivalence(self):
        eulers = euler.create_from_x_rotation(np.pi / 2.)
        m = matrix33.create_from_x_rotation(np.pi / 2.)
        q = quaternion.create_from_x_rotation(np.pi / 2.)
        qm = matrix33.create_from_quaternion(q)
        em = matrix33.create_from_eulers(eulers)
        self.assertTrue(np.allclose(qm, m))
        self.assertTrue(np.allclose(qm, em))
        self.assertTrue(np.allclose(m, em))

    def test_quaternion_matrix_conversion(self):
        # https://au.mathworks.com/help/robotics/ref/quat2rotm.html?requestedDomain=www.mathworks.com
        q = quaternion.create(0.7071, 0., 0., 0.7071)
        m33 = matrix33.create_from_quaternion(q)
        expected = np.array([
            [1., 0., 0.],
            [0.,-0.,-1.],
            [0., 1.,-0.],
        ])
        self.assertTrue(np.allclose(m33, expected))

        # issue #42
        q = quaternion.create(*[0.80087974, 0.03166748, 0.59114721,-0.09018753])
        m33 = matrix33.create_from_quaternion(q)
        q2 = quaternion.create_from_matrix(m33)
        print(q, q2)
        self.assertTrue(np.allclose(q, q2))

        q3 = quaternion.create_from_matrix(m33.T)
        self.assertFalse(np.allclose(q, q3))



if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""Provides functions to calculate and manipulate
Axis-Aligned Bounding Boxes (AABB).

AABB are a simple 3D rectangle with no orientation.
It is up to the user to provide translation.

An AABB is represented by an array of 2 x 3D vectors.
The first vector represents the minimum extent.
The second vector represents the maximum extent.

It should be noted that rotating the object within
an AABB will invalidate the AABB.
It is up to the user to either:

    * recalculate the AABB.
    * use an AAMBB instead.

TODO: add transform( matrix )
"""
from __future__ import absolute_import, division, print_function
import numpy as np
from .utils import all_parameters_as_numpy_arrays, parameters_as_numpy_arrays


class index:
    #: The index of the minimum vector within the AABB
    minimum = 0

    #: The index of the maximum vector within the AABB
    maximum = 1


def create_zeros(dtype=None):
    return np.zeros((2,3), dtype=dtype)

@parameters_as_numpy_arrays('min', 'max')
def create_from_bounds(min, max, dtype=None):
    """Creates an AABB using the specified minimum
    and maximum values.
    """
    dtype = dtype or min.dtype
    return np.array([min, max], dtype=dtype)

@parameters_as_numpy_arrays('points')
def create_from_points(points, dtype=None):
    """Creates an AABB from the list of specified points.

    Points must be a 2D list. Ie::
        numpy.array([
            [ x, y, z ],
            [ x, y, z ],
            ])
    """
    dtype = dtype or points.dtype
    return np.array(
        [
            np.amin(points, axis=0),
            np.amax(points, axis=0)
        ],
        dtype=dtype
    )

@parameters_as_numpy_arrays('aabbs')
def create_from_aabbs(aabbs, dtype=None):
    """Creates an AABB from a list of existing AABBs.

    AABBs must be a 2D list. Ie::
        numpy.array([
            AABB,
            AABB,
            ])
    """
    dtype = dtype or aabbs.dtype
    # reshape the AABBs as a series of points
    points = aabbs.reshape((-1, 3))

    return create_from_points(points, dtype)

@parameters_as_numpy_arrays('aabb')
def add_points(aabb, points):
    """Extends an AABB to encompass a list
    of points.
    """
    # find the minimum and maximum point values
    minimum = np.amin(points, axis=0)
    maximum = np.amax(points, axis=0)

    # compare to existing AABB
    return np.array(
        [
            np.minimum(aabb[0], minimum),
            np.maximum(aabb[1], maximum)
        ],
        dtype=aabb.dtype
    )

@parameters_as_numpy_arrays( 'aabbs' )
def add_aabbs(aabb, aabbs):
    """Extend an AABB to encompass a list
    of other AABBs.
    """
    # convert to points and use our existing add_points
    # function
    points = aabbs.reshape((-1, 3))

    return add_points(aabb, points)

@all_parameters_as_numpy_arrays
def centre_point(aabb):
    """Returns the centre point of the AABB.
    """
    return (aabb[0] + aabb[1]) * 0.5

@all_parameters_as_numpy_arrays
def minimum(aabb):
    """Returns the minimum point of the AABB.
    """
    return aabb[0].copy()

@all_parameters_as_numpy_arrays
def maximum(aabb):
    """ Returns the maximum point of the AABB.
    """
    return aabb[1].copy()

@all_parameters_as_numpy_arrays
def clamp_points(aabb, points):
    """Takes a list of points and modifies them to
    fit within the AABB.
    """
    return np.clip(points, a_min=aabb[0], a_max=aabb[1])




try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import sphere


class test_sphere(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.sphere
        from pyrr import sphere

    def test_create_from_points(self):
        # the biggest should be 5,5,5
        result = sphere.create_from_points([
            [ 0.0, 0.0, 0.0 ],
            [ 5.0, 5.0, 5.0 ],
            [ 0.0, 0.0, 5.0 ],
            [-5.0, 0.0, 0.0 ],
        ])
        # centred around 0,0,0
        # with MAX LENGTH as radius
        np.testing.assert_almost_equal(result, [0.,0.,0., 8.66025], decimal=5)

    def test_position(self):
        result = sphere.position([1.,2.,3.,4.])
        np.testing.assert_almost_equal(result, [1.,2.,3.], decimal=5)

    def test_radius(self):
        result = sphere.radius([1.,2.,3.,4.])
        np.testing.assert_almost_equal(result, 4., decimal=5)


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import vector4


class test_vector4(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.vector4
        from pyrr import vector4

    def test_create(self):
        result = vector4.create()
        np.testing.assert_almost_equal(result, [0.,0.,0.,0.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_values(self):
        result = vector4.create(1.,2.,3.,4., dtype=np.float32)
        np.testing.assert_almost_equal(result, [1.,2.,3.,4.], decimal=5)
        self.assertTrue(result.dtype == np.float32)

    def test_create_list(self):
        with self.assertRaises(ValueError):
            vector4.create([1., 2., 3., 4.])

    def test_create_unit_length_x(self):
        result = vector4.create_unit_length_x()
        np.testing.assert_almost_equal(result, [1.,0.,0.,0.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_unit_length_x_dtype(self):
        result = vector4.create_unit_length_x(dtype=np.float32)
        np.testing.assert_almost_equal(result, [1.,0.,0.,0.], decimal=5)
        self.assertTrue(result.dtype == np.float32)

    def test_create_unit_length_y(self):
        result = vector4.create_unit_length_y()
        np.testing.assert_almost_equal(result, [0.,1.,0.,0.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_unit_length_y_dtype(self):
        result = vector4.create_unit_length_y(dtype=np.float32)
        np.testing.assert_almost_equal(result, [0.,1.,0.,0.], decimal=5)
        self.assertTrue(result.dtype == np.float32)

    def test_create_unit_length_z(self):
        result = vector4.create_unit_length_z()
        np.testing.assert_almost_equal(result, [0.,0.,1.,0.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_unit_length_z_dtype(self):
        result = vector4.create_unit_length_z(dtype=np.float32)
        np.testing.assert_almost_equal(result, [0.,0.,1.,0.], decimal=5)
        self.assertTrue(result.dtype == np.float32)

    def test_create_unit_length_w(self):
        result = vector4.create_unit_length_w()
        np.testing.assert_almost_equal(result, [0.,0.,0.,1.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_unit_length_w_dtype(self):
        result = vector4.create_unit_length_w(dtype=np.float32)
        np.testing.assert_almost_equal(result, [0.,0.,0.,1.], decimal=5)
        self.assertTrue(result.dtype == np.float32)

    def test_create_from_matrix44_translation(self):
        mat = np.array([
            [1.,2.,3.,4.,],
            [5.,6.,7.,8.,],
            [9.,10.,11.,12.,],
            [13.,14.,15.,16.,],
        ])
        result = vector4.create_from_matrix44_translation(mat)
        np.testing.assert_almost_equal(result, [13.,14.,15.,16.], decimal=5)
        self.assertTrue(result.dtype == np.float)

    def test_create_from_matrix44_translation_dtype_matches(self):
        mat = np.array([
            [1.,2.,3.,4.,],
            [5.,6.,7.,8.,],
            [9.,10.,11.,12.,],
            [13.,14.,15.,16.,],
        ], dtype=np.float32)
        result = vector4.create_from_matrix44_translation(mat)
        np.testing.assert_almost_equal(result, [13.,14.,15.,16.], decimal=5)
        self.assertTrue(result.dtype == np.float32)

    def test_normalise_single_vector(self):
        result = vector4.normalise([1.,1.,1.,1.])
        np.testing.assert_almost_equal(result, [0.5, 0.5, 0.5, 0.5], decimal=5)

    def test_normalise_batch(self):
        result = vector4.normalise([
            [1.,1.,1.,1.],
            [-1.,-1.,-1.,1.],
            [0.,2.,7.,1.],
        ])
        expected = [
            [0.5, 0.5, 0.5, 0.5],
            [-0.5,-0.5,-0.5, 0.5],
            [0., 0.27216553, 0.95257934, 0.13608276],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_squared_length_single_vector(self):
        result = vector4.squared_length([1.,1.,1.,1.])
        np.testing.assert_almost_equal(result, 4., decimal=5)

    def test_squared_length_batch(self):
        result = vector4.squared_length([
            [1.,1.,1.,1.],
            [-1.,-1.,-1.,1.],
            [0.,2.,7.,1.],
        ])
        expected = [
            4.,
            4.,
            54.,
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_length(self):
        result = vector4.length([1.,1.,1.,1.])
        np.testing.assert_almost_equal(result, 2., decimal=5)

    def test_length_batch(self):
        result = vector4.length([
            [1.,1.,1.,1.],
            [-1.,-1.,-1.,1.],
            [0.,2.,7.,1.],
        ])
        expected = [
            2.,
            2.,
            7.34846923,
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_set_length(self):
        result = vector4.set_length([1.,1.,1.,1.],3.)
        expected = [1.5,1.5,1.5,1.5]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_set_length_batch_vector(self):
        result = vector4.set_length([
            [1.,1.,1.,1.],
            [-1.,-1.,-1.,1.],
            [0.,2.,7.,1.],
            ], 2.0)
        expected = [
            [1.,1.,1.,1.],
            [-1.,-1.,-1.,1.],
            [0., 0.54433105, 1.90515869, 0.27216553],
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)

    def test_dot_adjacent(self):
        result = vector4.dot([1.,0.,0.,1.], [0.,1.,0.,1.])
        np.testing.assert_almost_equal(result, 1.0, decimal=5)

    def test_dot_parallel(self):
        result = vector4.dot([0.,1.,0.,1.], [0.,1.,0.,1.])
        np.testing.assert_almost_equal(result, 2.0, decimal=5)

    def test_dot_angle(self):
        result = vector4.dot([.2,.2,0.,1.], [2.,-.2,0.,1.])
        np.testing.assert_almost_equal(result, 1.359999, decimal=5)

    def test_dot_batch(self):
        result = vector4.dot([
            [1.,0.,0.,1.],
            [0.,1.,0.,1.],
            [.2,.2,0.,1.]
        ],[
            [0.,1.,0.,1.],
            [0.,1.,0.,1.],
            [2.,-.2,0.,1.]
        ])
        expected = [
            1.,
            2.,
            1.36
        ]
        np.testing.assert_almost_equal(result, expected, decimal=5)


if __name__ == '__main__':
    unittest.main()



try:
    import unittest2 as unittest
except:
    import unittest
import numpy as np
from pyrr import geometric_tests as gt
from pyrr import line, plane, ray, sphere

class test_geometric_tests(unittest.TestCase):
    def test_import(self):
        import pyrr
        pyrr.geometric_tests
        from pyrr import geometric_tests

    def test_point_intersect_line(self):
        p = np.array([1.,1.,1.])
        l = np.array([[0.,0.,0.],[2.,2.,2.]])
        result = gt.point_intersect_line(p, l)
        self.assertTrue(np.array_equal(result, p))

    def test_point_intersect_line_invalid(self):
        p = np.array([3.,3.,3.])
        l = np.array([[0.,0.,0.],[2.,2.,2.]])
        result = gt.point_intersect_line(p, l)
        self.assertTrue(np.array_equal(result, p))

    def test_point_intersect_line_segment(self):
        p = np.array([1.,1.,1.])
        l = np.array([[0.,0.,0.],[2.,2.,2.]])
        result = gt.point_intersect_line_segment(p, l)
        self.assertTrue(np.array_equal(result, p))

    def test_point_intersect_line_segment_invalid(self):
        p = np.array([3.,3.,3.])
        l = np.array([[0.,0.,0.],[2.,2.,2.]])
        result = gt.point_intersect_line_segment(p, l)
        self.assertEqual(result, None)

    def test_point_intersect_rectangle_valid_intersections_1(self):
        r = np.array([
            [0.0, 0.0],
            [5.0, 5.0]
        ])
        p = [ 0.0, 0.0]
        result = gt.point_intersect_rectangle(p, r)
        self.assertTrue(np.array_equal(result, p))

    def test_point_intersect_rectangle_valid_intersections_2(self):
        r = np.array([
            [0.0, 0.0],
            [5.0, 5.0]
        ])
        p = [ 5.0, 5.0]
        result = gt.point_intersect_rectangle(p, r)
        self.assertTrue(np.array_equal(result, p))

    def test_point_intersect_rectangle_valid_intersections_3(self):
        r = np.array([
            [0.0, 0.0],
            [5.0, 5.0]
        ])
        p = [ 1.0, 1.0]
        result = gt.point_intersect_rectangle(p, r)
        self.assertTrue(np.array_equal(result, p))

    def test_point_intersect_rectangle_invalid_intersections_1(self):
        r = np.array([
            [0.0, 0.0],
            [5.0, 5.0]
        ])

        p = [-1.0, 1.0]
        result = gt.point_intersect_rectangle(p, r)
        self.assertFalse(np.array_equal(result, p))

    def test_point_intersect_rectangle_invalid_intersections_2(self):
        r = np.array([
            [0.0, 0.0],
            [5.0, 5.0]
        ])
        p = [ 1.0, 10.0]
        result = gt.point_intersect_rectangle(p, r)
        self.assertFalse(np.array_equal(result, p))

    def test_point_intersect_rectangle_invalid_intersections_3(self):
        rect = np.array([
            [0.0, 0.0],
            [5.0, 5.0]
        ])
        point = [ 1.0,-1.0]
        result = gt.point_intersect_rectangle(point, rect)
        self.assertFalse(np.array_equal(result, point))

    def test_ray_intersect_plane(self):
        r = ray.create([0.,-1.,0.],[0.,1.,0.])
        p = plane.create([0.,1.,0.], 0.)
        result = gt.ray_intersect_plane(r, p)
        self.assertFalse(np.array_equal(result, [0.,1.,0.]))

    def test_ray_intersect_plane_front_only(self):
        r = ray.create([0.,-1.,0.],[0.,1.,0.])
        p = plane.create([0.,1.,0.], 0.)
        result = gt.ray_intersect_plane(r, p, front_only=True)
        self.assertEqual(result, None)

    def test_ray_intersect_plane_invalid(self):
        r = ray.create([0.,-1.,0.],[1.,0.,0.])
        p = plane.create([0.,1.,0.], 0.)
        result = gt.ray_intersect_plane(r, p)
        self.assertEqual(result, None)

    def test_point_closest_point_on_ray(self):
        l = line.create_from_points(
            [ 0.0, 0.0, 0.0 ],
            [10.0, 0.0, 0.0 ]
        )
        p = np.array([ 0.0, 1.0, 0.0])
        result = gt.point_closest_point_on_ray(p, l)
        self.assertTrue(np.array_equal(result, [ 0.0, 0.0, 0.0]))

    def test_point_closest_point_on_line(self):
        p = np.array([0.,1.,0.])
        l = np.array([[0.,0.,0.],[2.,0.,0.]])
        result = gt.point_closest_point_on_line(p, l)
        self.assertTrue(np.array_equal(result, [0.,0.,0.]), (result,))

    def test_point_closest_point_on_line_2(self):
        p = np.array([3.,0.,0.])
        l = np.array([[0.,0.,0.],[2.,0.,0.]])
        result = gt.point_closest_point_on_line(p, l)
        self.assertTrue(np.array_equal(result, [3.,0.,0.]), (result,))

    def test_point_closest_point_on_line_segment(self):
        p = np.array([0.,1.,0.])
        l = np.array([[0.,0.,0.],[2.,0.,0.]])
        result = gt.point_closest_point_on_line_segment(p, l)
        self.assertTrue(np.array_equal(result, [0.,0.,0.]), (result,))

    def test_vector_parallel_vector(self):
        v1 = np.array([1.,0.,0.])
        v2 = np.array([2.,0.,0.])
        self.assertTrue(gt.vector_parallel_vector(v1,v2))

    def test_vector_parallel_vector_invalid(self):
        v1 = np.array([1.,0.,0.])
        v2 = np.array([0.,1.,0.])
        self.assertTrue(False == gt.vector_parallel_vector(v1,v2))

    def test_ray_parallel_ray(self):
        r1 = ray.create([0.,0.,0.],[1.,0.,0.])
        r2 = ray.create([1.,0.,0.],[2.,0.,0.])
        self.assertTrue(gt.ray_parallel_ray(r1,r2))

    def test_ray_parallel_ray_2(self):
        r1 = ray.create([0.,0.,0.],[1.,0.,0.])
        r2 = ray.create([1.,0.,0.],[0.,1.,0.])
        self.assertTrue(False == gt.ray_parallel_ray(r1,r2))

    def test_ray_parallel_ray_3(self):
        r1 = ray.create([0.,0.,0.],[1.,0.,0.])
        r2 = ray.create([0.,1.,0.],[1.,0.,0.])
        self.assertTrue(gt.ray_parallel_ray(r1,r2))

    def test_ray_coincident_ray(self):
        r1 = ray.create([0.,0.,0.],[1.,0.,0.])
        r2 = ray.create([1.,0.,0.],[2.,0.,0.])
        self.assertTrue(gt.ray_coincident_ray(r1,r2))

    def test_ray_coincident_ray_2(self):
        r1 = ray.create([0.,0.,0.],[1.,0.,0.])
        r2 = ray.create([1.,0.,0.],[0.,1.,0.])
        self.assertTrue(False == gt.ray_coincident_ray(r1,r2))

    def test_ray_coincident_ray_3(self):
        r1 = ray.create([0.,0.,0.],[1.,0.,0.])
        r2 = ray.create([0.,1.,0.],[1.,0.,0.])
        self.assertTrue(False == gt.ray_coincident_ray(r1,r2))

    def test_ray_intersect_aabb_valid_1(self):
        a = np.array([[-1.0,-1.0,-1.0], [ 1.0, 1.0, 1.0]])
        r = np.array([[ 0.5, 0.5, 0.0], [ 0.0, 0.0,-1.0]])
        result = gt.ray_intersect_aabb(r, a)
        self.assertTrue(np.array_equal(result, [ 0.5, 0.5,-1.0]))

    def test_ray_intersect_aabb_valid_2(self):
        a = np.array([[-1.0,-1.0,-1.0], [ 1.0, 1.0, 1.0]])
        r = np.array([[2.0, 2.0, 2.0], [ -1.0, -1.0, -1.0]])
        result = gt.ray_intersect_aabb(r, a)
        self.assertTrue(np.array_equal(result, [1.0, 1.0, 1.0]))

    def test_ray_intersect_aabb_invalid_1(self):
        a = np.array([[-1.0,-1.0,-1.0], [ 1.0, 1.0, 1.0]])
        r = np.array([[2.0, 2.0, 2.0], [ 1.0, 1.0, 1.0]])
        result = gt.ray_intersect_aabb(r, a)
        self.assertEqual(result, None)

    def test_point_height_above_plane(self):
        v1 = np.array([ 0.0, 0.0, 1.0])
        v2 = np.array([ 1.0, 0.0, 1.0])
        v3 = np.array([ 0.0, 1.0, 1.0])
        p = np.array([0.0, 0.0, 20.0])

        pl = plane.create_from_points(v1, v2, v3)
        pl = plane.invert_normal(pl)

        result = gt.point_height_above_plane(p, pl)
        self.assertEqual(result, 19.)

    def test_point_closest_point_on_plane(self):
        pl = np.array([ 0.0, 1.0, 0.0, 0.0])
        p = np.array([ 5.0, 20.0, 5.0])
        result = gt.point_closest_point_on_plane(p, pl)
        self.assertTrue(np.array_equal(result, [ 5.0, 0.0, 5.0]))

    def test_sphere_does_intersect_sphere_1(self):
        s1 = sphere.create()
        s2 = sphere.create()
        self.assertTrue(gt.sphere_does_intersect_sphere(s1, s2))

    def test_sphere_does_intersect_sphere_2(self):
        s1 = sphere.create()
        s2 = sphere.create([1.,0.,0.])
        self.assertTrue(gt.sphere_does_intersect_sphere(s1, s2))

    def test_sphere_does_intersect_sphere_3(self):
        s1 = sphere.create()
        s2 = sphere.create([2.,0.,0.], 1.0)
        self.assertTrue(gt.sphere_does_intersect_sphere(s1, s2))

    def test_sphere_does_intersect_sphere_4(self):
        s1 = sphere.create()
        s2 = sphere.create([2.,0.,0.], 0.5)
        self.assertTrue(False == gt.sphere_does_intersect_sphere(s1, s2))

    def test_sphere_penetration_sphere_1(self):
        s1 = sphere.create()
        s2 = sphere.create()
        self.assertEqual(gt.sphere_penetration_sphere(s1, s2), 2.0)

    def test_sphere_penetration_sphere_2(self):
        s1 = sphere.create()
        s2 = sphere.create([1.,0.,0.], 1.0)
        self.assertEqual(gt.sphere_penetration_sphere(s1, s2), 1.0)

    def test_sphere_penetration_sphere_3(self):
        s1 = sphere.create()
        s2 = sphere.create([2.,0.,0.], 1.0)
        self.assertEqual(gt.sphere_penetration_sphere(s1, s2), 0.0)

    def test_sphere_penetration_sphere_4(self):
        s1 = sphere.create()
        s2 = sphere.create([3.,0.,0.], 1.0)
        self.assertEqual(gt.sphere_penetration_sphere(s1, s2), 0.0)

if __name__ == '__main__':
    unittest.main()

